"""Extraction prompts for conversation processing."""

from .conversation import (
    CONVERSATION_EXTRACTION_SYSTEM_PROMPT,
    CONVERSATION_EXTRACTION_USER_PROMPT,
    DEDUP_DECISION_FUNCTIONS,
    DEDUP_DECISION_PROMPT,
)

__all__ = [
    "CONVERSATION_EXTRACTION_SYSTEM_PROMPT",
    "CONVERSATION_EXTRACTION_USER_PROMPT",
    "DEDUP_DECISION_PROMPT",
    "DEDUP_DECISION_FUNCTIONS",
]
